/**
 *****************************************************************************
   @example  ADT7410I2C.c
   @brief    Set of functions to interface ADuCM360/61 to ADT7410 via I2C.
	           The I2C peripheral shoud be initialised before using these functions.

   @version  V0.1
   @author   ADI
   @date     July 2013 
              

All files for ADuCM360/361 provided by ADI, including this file, are
provided  as is without warranty of any kind, either expressed or implied.
The user assumes any and all risk from the use of this code.
It is the responsibility of the person integrating this code into an application
to ensure that the resulting application performs as required and is safe.

**/

#include <ADuCM360.h>
#include <..\common\I2cLib.h>
#include "ADT7410I2C.h"


unsigned int uiMasterRxIndex = 0;
unsigned int uiMasterTxIndex = 0;
unsigned char ucMasterTxDat[2] = {0x00,0x00};
unsigned char ucMasterRxDat[4] = {0,0,0,0};
unsigned char ucComplete = 0;
unsigned char Rxdone =0;
void delay(int);


/** 
   @fn ADT7410_I2CCFG(char Address,int Resolution,int iMode,int CT,int INT,int INTCTmode,int Faultqueue)
   @brief Configure the temperature sensor resolution, mode, CT pin polarity, INT pin polarity etc
   @param Address :{ADT7410_ADDRESS} 
   @param Resolution :{BIT16, BIT13} 
         - select the resolution of the ADC:
         - BIT16 for 16 bit resolution
         - BIT13 for 13 bit resolution
   @param iMode :{ADT7410_OP_MODE_CONT_CONV, ADT7410_OP_MODE_ONE_SHOT, ADT7410_OP_MODE_1_SPS, ADT7410_OP_MODE_SHUTDOWN} 
         - select the operational mode of the ADT7310:
         - ADT7410_OP_MODE_CONT_CONV for continuous mode
         - ADT7410_OP_MODE_ONE_SHOT for one shot mode
         - ADT7410_OP_MODE_1_SPS for 1SPS mode
         - ADT7410_OP_MODE_SHUTDOWN for shutdown mode
   @param CT :{Active_Low, Active_High} 
         - select the output polarity of the CT pin:
         - Active_Low for active low
         - Active_High for active high
   @param INT :{Active_Low, Active_High} 
         - select the output polarity of the INT pin:
         - Active_Low for active low
         - Active_High for active high
   @param INTCTmode :{interrupt_m, comparator_m} 
         - selects between comparator mode and interrupt mode: 
         - interrupt_m
         - comparator_m
   @param Fault_queue :{ADT7410_FAULT_QUEUE_1_FAULT, ADT7410_FAULT_QUEUE_2_FAULTS, ADT7410_FAULT_QUEUE_3_FAULTS, ADT7410_FAULT_QUEUE_4_FAULTS}
         - set the number of overtemperature/undertemperature faults that can occur before setting the INT and CT pins.
         - ADT7410_FAULT_QUEUE_1_FAULT     
         - ADT7410_FAULT_QUEUE_2_FAULT    
         - ADT7410_FAULT_QUEUE_3_FAULT     
         - ADT7410_FAULT_QUEUE_4_FAULT     
   @pre     I2C must be configured before this function is called.
   @return  1
**/
int ADT7410_I2CCFG(char Address,int Resolution,int iMode,int CT,int INT,int INTCTmode,int Faultqueue)
{
  unsigned char settingbits;

  settingbits = Faultqueue << 1;
  settingbits |= CT << 2;
  settingbits |= INT << 3;
  settingbits |= INTCTmode << 4;
  settingbits |= (iMode&2) << 5;
  settingbits |= Resolution << 7;
  ucMasterTxDat[0] = 0x03;
  ucMasterTxDat[1] = settingbits;
  
  uiMasterTxIndex = 0;     
  I2cTx(MASTER,ucMasterTxDat[uiMasterTxIndex++]);            		
  I2cMWrCfg((Address<<1));
	 
  while (!ucComplete){}
  ucComplete = 0;
    
  return 1;
}

/** 
   @fn ADT7410_I2CID(char Address)
   @brief Return sensor ID
   @param Address :{ADT7410_ADDRESS}       
   @return Return sensor ID
**/
int ADT7410_I2CID(char Address)
{
  unsigned long Rxdat = 0;

  uiMasterRxIndex = 0;
  Rxdat = 0;
  I2cFifoFlush(MASTER,ENABLE);
  I2cFifoFlush(MASTER, DISABLE);
  NVIC_DisableIRQ(I2CM_IRQn); 
  I2cMWrCfg((Address << 1));
  I2cTx(MASTER,0x0B);
  while((I2cSta(MASTER)&I2CMSTA_TXFSTA_MSK)!= 0x0){} //repeated start condition
  I2cMRdCfg((Address<<1),1,DISABLE);
  while((I2cSta(MASTER)& I2CMSTA_RXREQ) != I2CMSTA_RXREQ){} //wait for a byte to be received
  Rxdat = I2cRx(MASTER);
  I2cFifoFlush(MASTER,ENABLE);
  I2cFifoFlush(MASTER,DISABLE);
  NVIC_EnableIRQ(I2CM_IRQn);		
  return Rxdat;
}

/** 
   @fn ADT7410_Reset(void)
   @brief Reset the ADT7410
   @return 1
**/
int ADT7410_I2CReset(char Address)                     
{
  I2cMWrCfg((Address << 1));                       //To reset the ADT7410 without having to reset the entire I2C bus
  I2cTx(MASTER,0x2F);                                
  while((I2cSta(MASTER)&I2CMSTA_TXFSTA_MSK)!= 0x0){} //repeated start condition
  I2cMRdCfg((Address<<1),1,DISABLE);            		
  delay(0x3200);                                     //  Does not respond to the I2C bus during the default values upload                                              //   for approximately 200us.				
  return 1;
}

/** 
   @fn ADT7410_I2C_T_Setpoint(char BusAddress,char REGadd,int RorW,int Value)
   @brief Configure or read back one of the four set point register (THIGH,TLOW,TCRIT or THYST)
   @param BusAddress :{ADT7410_ADDRESS} 
   @param REG_Address :{T_Crit_Setpoint, T_Hyst_Setpoint, T_High_Setpoint, T_Low_Setpoint} 
         - select the register to write:
         - T_Crit_Setpoint
         - T_Hyst_Setpoint
         - T_High_Setpoint 
         - T_Low_Setpoint  
   @param RorW :{Read, Write} 
         - select the operation:
         - Read
         - Write 
   @param value :{0-256}     
   @return 1
**/
int ADT7410_I2C_T_Setpoint(char BusAddress,char REGadd,int RorW,int Value)
{     
  unsigned char Rxdata = 0; 	

  if(RorW)
  {
    I2cFifoFlush(MASTER,ENABLE);
    I2cFifoFlush(MASTER,DISABLE);
    NVIC_DisableIRQ(I2CM_IRQn);
    I2cTx(MASTER,REGadd);
    I2cMWrCfg((BusAddress << 1));
    while((I2cSta(MASTER)&I2CMSTA_TXFSTA_MSK)!= 0x0){} //repeated start condition
    I2cMRdCfg((BusAddress<<1),2,DISABLE);
    while((I2cSta(MASTER)& I2CMSTA_RXREQ) != I2CMSTA_RXREQ){} //wait for a byte to be received
    Rxdata = I2cRx(MASTER);
    I2cFifoFlush(MASTER,ENABLE);
    I2cFifoFlush(MASTER,DISABLE);
    NVIC_EnableIRQ(I2CM_IRQn);
    return Rxdata;
  }
  else
  {
    uiMasterTxIndex = 0; 
    I2cFifoFlush(MASTER,ENABLE);
    I2cFifoFlush(MASTER,DISABLE);
    NVIC_EnableIRQ(I2CM_IRQn);
    ucMasterTxDat[0] = REGadd;
    ucMasterTxDat[1] = Value;    
    I2cTx(MASTER,ucMasterTxDat[uiMasterTxIndex++]);            		
    I2cMWrCfg((BusAddress<<1));
    while (!ucComplete){}
    ucComplete = 0;
    return 0;
  }	
}

/** 
   @fn ADT7410_I2CGetTemperature(char Address,unsigned char *Status,unsigned char *Config)
   @brief Return temperature measurement in degree Celsius. This function also updates the status and configuration variables.
   @param Address :{ADT7410_ADDRESS} 
   @param *Status :{} 
         - variable 
   @param *Config :{} 
         - variable
   @return float temperature in degree Celsius
**/
float ADT7410_I2CGetTemperature(char Address,unsigned char *Status,unsigned char *Config)
{
  unsigned char msbTemp = 0;
  unsigned char lsbTemp = 0;
  unsigned short temp = 0;
  float tempC = 0;
  unsigned char GetAdd = (Address << 1)+1;
 
  uiMasterRxIndex = 0;
  I2cMRdCfg(GetAdd,sizeof(ucMasterRxDat),DISABLE);
  while (!Rxdone){}
  Rxdone = 0;
  msbTemp = ucMasterRxDat[0];
  lsbTemp = ucMasterRxDat[1];
  *Status = ucMasterRxDat[2];
  *Config = ucMasterRxDat[3];
  temp = ((unsigned short)msbTemp << 8) + lsbTemp;		
  if(ucMasterRxDat[3]&(1<<7))
  {
    if(temp & 0x8000)                       // Negative temperature 
    {
      tempC = (float)((signed long)temp - 65536) / 128;
    }
    else
    {
      tempC = (float)temp / 128;      // Positive temperature 
    }
  }
  else
  {
    temp >>= 3;
    if(temp & 0x1000)                     // Negative temperature
    {
      tempC = (float)((signed long)temp - 8192) / 16;
    }
    else                // Positive temperature
    {
      tempC = (float)temp / 16;
    }
  }
  return tempC;
}

// Simple Delay routine
void delay (int length)
{
  while (length >0)
    length--;
}

void I2C0_Master_Int_Handler(void) 
{
  unsigned int uiStatus;
  uiStatus = I2cSta(MASTER);

  if((uiStatus & I2CMSTA_RXREQ) == I2CMSTA_RXREQ)	// Master Recieve IRQ
  {
    ucMasterRxDat[uiMasterRxIndex++] = I2cRx(MASTER);
    if(uiMasterRxIndex > (sizeof(ucMasterRxDat)-1) )
    { 			 // Resetting value of i if it has been incremented by RX
      Rxdone = 1; 
      uiMasterRxIndex = 0;
    }
  }
  if((uiStatus & I2CMSTA_TXREQ) == I2CMSTA_TXREQ) // Master Transmit IRQ	
  {
    if (uiMasterTxIndex < sizeof(ucMasterTxDat) )// configuration not complete
      I2cTx(MASTER, ucMasterTxDat[uiMasterTxIndex++]);
    else // configuration complete
    {
      I2cMCfg(I2CMCON_TXDMA_DIS|I2CMCON_RXDMA_DIS, I2CMCON_IENCMP|I2CMCON_IENRX|I2CMCON_IENTX_DIS, I2CMCON_MAS_EN);    // TXREQ disabled to avoid multiple unecessary interrupts   
    }
  }	
  if((uiStatus & I2CMSTA_TCOMP_SET) == I2CMSTA_TCOMP_SET) // communication complete	
  {
    I2cMCfg(I2CMCON_TXDMA_DIS|I2CMCON_RXDMA_DIS, I2CMCON_IENCMP|I2CMCON_IENRX|I2CMCON_IENTX_EN, I2CMCON_MAS_EN);   // TXREQ enabled for future master transmissions    
    ucComplete = 1;
  }
}
